# Copyright 2005 by
#           Atte Timonen
#           Tarmo Toikkanen
#           Teemu Leinonen
#           Lily Diaz
#           Janne Pietarila
#
# Code snippets used from Fle3 (fle3.uiah.fi),
# credits go to the Fle3 Team.
#
# This file is part of the ImaNote product.
#
# ImaNote is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# ImaNote is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ImaNote; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

# Python
import re
import time
# Zope
import DateTime
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from Acquisition import Implicit
from Products.ZCatalog.ZCatalog import ZCatalog
from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2
from OFS.Folder import Folder
#import Acquisition
# Own
#from TiledImage import *
from Annotation import Annotation
from common import *
from Globals import Persistent

class AnnotationManager(BTreeFolder2, Persistent, Implicit):
    """A manager for annotations """
    meta_type="AnnotationManager"
    security = ClassSecurityInfo()
    security.declareObjectPublic()
    def __init__(self, id):
      """ Constructs. """
      self.id=id
      BTreeFolder2.__init__(self)
      catalog = ZCatalog('catalog_annotations', 'ZCatalog for annotations')
      catalog.addIndex('getTitle', 'TextIndex')
      catalog.addIndex('getData', 'TextIndex')
      catalog.addIndex('getAllData', 'TextIndex')
      catalog.addIndex('getAuthor', 'FieldIndex')
      catalog.addIndex('getMediaType', 'FieldIndex')
      catalog.addIndex('getKeywords', 'TextIndex')
      self._setObject('catalog_annotations', catalog)

        
    security.declarePrivate('removeAnnotation')
    def removeAnnotation(self, a_id):
      if a_id in self.objectIds(Annotation.meta_type):
        self._getOb(a_id).unindex_object()
        self._delObject(a_id)

    security.declarePrivate('addAnnotation')
    def addAnnotation(self, annotation):
      self._setObject(annotation.getId(), annotation)
      
    security.declareProtected(PERM_MANAGE, 'getAnnotations')
    def getAnnotations(self):
      annos=[]
      for (id, anno) in self.objectItems(Annotation.meta_type):
        if anno.mayView():
          annos.append((id, anno))
      return annos
    
    security.declarePrivate('getAnnotationById')
    def getAnnotationById(self, id):
      anno=self.get(id)
      if anno and anno.mayView():
        return anno
      return None
      
    
InitializeClass(AnnotationManager)
