# Copyright 2005 by
#           Atte Timonen
#           Tarmo Toikkanen
#           Teemu Leinonen
#           Lily Diaz
#           Janne Pietarila
#
# Code snippets used from Fle3 (fle3.uiah.fi),
# credits go to the Fle3 Team.
#
# This file is part of the ImaNote product.
#
# ImaNote is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# ImaNote is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ImaNote; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

#from Utils import *
import AccessControl
from common import *
import OFS
import OFS.SimpleItem
import OFS.ObjectManager
from OFS.Folder import Folder

class Users(
        Folder,
        OFS.SimpleItem.Item,
        OFS.ObjectManager.ObjectManager,
        AccessControl.Role.RoleManager
        ):
    """User management container."""
    #meta_type='Users'
    security=AccessControl.ClassSecurityInfo()

    def __init__(self,id,title):
      self.id=id
      self.title=title

    def manage_afterAdd(self,item,container):
        #self.manage_permission(PERM_MANAGE, role_staff, 1)
        #self.manage_permission(PERM_EDIT, role_staff, 1)
        pass

    # Adds user, called from code
    security.declareProtected(PERM_MANAGE, 'add_user')
    def add_user(self,uname,password):
        """Add new user."""
        ob = UserInfo(uname,password)
        self._setObject(uname,ob)
        ob.__of__(self).confirm_account()

    security.declareProtected(PERM_VIEW, 'add_user_request')
    def add_user_request(self,uname,password,password2, REQUEST=None):
        """Add new user application."""
        names = {'uname': 'Account name',
                 'password': 'Password',
                 'password2': 'Password confirmation'}
        optionals = {}
        other_errors={}
        if self.acl_users.getUser(REQUEST['uname']):
          raise 'Error', 'Account already exists'
        if REQUEST['password']!=REQUEST['password2']:
          raise 'Error', "Passwords don't match"
        self.add_user_impl(REQUEST['uname'], REQUEST['password'],
            REQUEST['password2'])

        if REQUEST:
            REQUEST.RESPONSE.redirect(self.absolute_url())

    # Adds user, called from add_user_request
    def add_user_impl(self,uname,password,password2):
        if uname in self.objectIds('UserInfo'):
            raise 'Error','Username %s already used!' % uname
        ob = UserInfo(uname,password)
        self._setObject(uname,ob)
        ob.__of__(self).confirm_account()

    security.declareProtected(PERM_MANAGE, 'get_user_names')
    def get_user_names(self):
        """Return list of all users."""
        return self.acl_users.getUserNames()

    security.declareProtected(PERM_MANAGE, 'get_users')
    def get_users(self):
        """Return list of all users."""
        users=[]
        for i in self.get_user_names():
          users.append(self.acl_users.getUser(i))
        return users

    security.declareProtected(PERM_MANAGE, 'manage_users_handler')
    def manage_users_handler(self, save=''):
      """Handle user management form."""
      if not save:
        return
      for i in self.get_users():
        if reallyHasattr(self.REQUEST, i.getId()+'deleteuser') and\
            getattr(self.REQUEST, i.getId()+'deleteuser')=='on':
          self.acl_users._doDelUsers((i.getUserName(),))
      self.REQUEST.RESPONSE.redirect('manageUsers.html')

class UserInfo(
    OFS.SimpleItem.Item,
    OFS.ObjectManager.ObjectManager,
    AccessControl.Role.RoleManager
    ):
    """Single user info object."""
    meta_type='UserInfo'
    security=AccessControl.ClassSecurityInfo()

    names = {'first': 'First name',
             'last': 'Last name'}
    optionals = {
                 'password':'Password',
                 'password2':'Password confirmation'}
    def __get_dict_copy(self,dict):
        d = {}
        for (key,val) in dict.items():
            d[key]=val
        return d


    def get_empty_user_info(self):
        info={}
        for key in UserInfo.names.keys():
            info[key]=''
        for key in UserInfo.optionals.keys():
            info[key]=''
        return info


    def __init__(self,uname,password):
        self.id=uname
        info = self.get_empty_user_info()
        self.__pass=password
        self.__info=info

    def manage_afterAdd(self,item,container):
        #self.manage_permission(PERM_EDIT, ('Owner',), 1)
        self.set_info(self.__info)
        del self.__info

    security.declareProtected(PERM_MANAGE, 'confirm_account')
    def confirm_account(self,REQUEST=None):
        """Confirm the account."""
        self.acl_users._doAddUser(self.id,self.__pass,[],())
        del self.__pass
        self.changeOwnership(self.acl_users.getUser(
            self.get_uname()).__of__(self.acl_users))
        self.manage_setLocalRoles(self.get_uname(),['Owner',])
        if REQUEST:
            REQUEST.RESPONSE.redirect(self.parent().absolute_url())

    security.declareProtected(PERM_VIEW, 'get_uname')
    def get_uname(self):
        """Get user account name."""
        return self.id

    def set_info(self,values):
        password=values['password']
        password2=values['password2']
        if password and password==password2:
            if hasattr(self.acl_users.getUser(self.id),'roles'):
                roles = self.acl_users.getUser(self.id).roles
            else:
                roles = ()
            self.acl_users._doChangeUser(
                self.id,
                password,
                roles,
                [])
