# Copyright 2005 by
#           Atte Timonen
#           Tarmo Toikkanen
#           Teemu Leinonen
#           Lily Diaz
#           Janne Pietarila
#
# Code snippets used from Fle3 (fle3.uiah.fi),
# credits go to the Fle3 Team.
#
# This file is part of the ImaNote product.
#
# ImaNote is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# ImaNote is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with ImaNote; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA


import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from TiledImage import *
from tests import * 
from Annotation import *

class TestAnnotations(ZopeTestCase.ZopeTestCase):    
    annoText='a0'
    annoLink='a1'
    annoBig='a2'
  
    def afterSetUp(self):
      id=made.generateId()
      made.testImage.addAnnotation(Annotation(0,0,10,10,id))
      made.testImage.getAnnotation(id).editData(text='testTextAnnotation')
      id=made.generateId()
      made.testImage.addAnnotation(Annotation(10,0,20,10,id))
      made.testImage.getAnnotation(id).editData(url='http://www.testLinkAnnotation.com')
      id=made.generateId()
      made.testImage.addAnnotation(Annotation(10,10,800,800,id))
      made.testImage.getAnnotation(id).editData(text='bigTextAnnotation')

    def testGet(self):
      assert made.testImage.getAnnotation(self.annoText).getText()==\
          'testTextAnnotation', 'getAnnotation didnt return correct annotation'
      
    def testOnMap(self):
      assert made.testImage.getAnnotationsForPoint(5,5),\
          "Text annotation not on map"
      assert made.testImage.getAnnotationsForPoint(15,5),\
          "Link annotation not on map"

    def testPosition(self):
      pos=made.testImage.getAnnotation(self.annoLink).getPosition(640,480,1)
      assert pos[0]==15 and pos[1]==5, "Annotation position invalid"
      assert made.testImage.getAnnotation(self.annoLink).doesCover(10,0),\
          "Annotation is supposed to cover area"
      assert not made.testImage.getAnnotation(self.annoBig).doesCover(5,5),\
          "Annotation is not supposed to cover area"
      assert made.testImage.getAnnotation(self.annoBig).getCoordinates()==\
          (10,10, 800, 800), 'Annotation coordinates incorrect'

    def testSize(self):
      assert made.testImage.getAnnotation(self.annoLink).getSize()==10*10,\
          "Annotation size not correct"
      assert made.testImage.getAnnotation(self.annoBig).getSize()==790*790,\
          "Annotation size not correct"

    def testType(self):
      assert made.testImage.getAnnotation(self.annoBig).getText()==\
          'bigTextAnnotation', 'Annotation text invalid'
      assert made.testImage.getAnnotation(self.annoLink).getUrl()==\
          'http://www.testLinkAnnotation.com', 'Annotation link invalid'
      assert made.testImage.getAnnotation(self.annoBig).getText(),\
          'Annotation type is not text'
      assert made.testImage.getAnnotation(self.annoLink).getUrl(),\
          'Annotation type is not link'
      assert not made.testImage.getAnnotation(self.annoLink).getText(),\
          'Annotation type is text, shouldnt be.'

    def testSearching(self):
      id=made.generateId()
      made.testImage.addAnnotation(Annotation(0,0,10,10,id))
      made.testImage.getAnnotation(id).editData(text='test Annotation Search')
      made.testImage.getAnnotation(id).editData(author='atte')
      assert made.annotations.testImage.catalog_annotations(
          getData='test Annotation')[0].getObject().getText()\
              =='test Annotation Search', 'Search results invalid'
      assert made.annotations.testImage.catalog_annotations(
          getData='search annotation')[0].getObject().getAuthor()\
              =='atte', 'Search results invalid'
      assert made.annotations.testImage.catalog_annotations(
          getAuthor='atte')[0].getObject().getText()\
              =='test Annotation Search', 'Search results invalid'
      assert len(made.annotations.testImage.\
              catalog_annotations(getAuthor='tte'))==0,\
              'Search returned results that it shouldn\'t have returned'
          
    def testData(self):
      id=made.generateId()
      x=Annotation(10,10,30,30,id)
      title='Hieno annotaatio'
      author='Matti meiklinen'
      url='http://locallhost/'
      urlName='A cool zope website'
      text='Great annotation'
      description='Wild action movie'
      coordinates=(20,20,45,45)
      keywords='test1 test2 "test3 test4 test5" test6 "test7 test8" "test9 test10" test11 "test12" test13'
      date1=x.getDate()
      assert x.getCoordinates()==(10,10,30,30), 'Annotation coordinates'+\
          'incorrect'
      x.editData(coordinates,title,author,url,urlName,text,description, keywords=keywords)
      assert x.getMimeType()=='text/plain', 'Annotation mimetype incorrect'
      assert x.getMediaType()=='text', 'Annotation media type incorrect'
      assert x.getCoordinates()==(20,20,45,45), 'Annotation coordinates'+\
          ' incorrect after edit: '
      assert date1!=x.getDate(), 'Editing data did not update date'
      assert x.getTitle()==title, 'Title not correct'
      assert x.getAuthor()==author, 'Author not correct'
      assert x.getUrl()==url, 'Url not correct'
      assert x.getText()==text, 'Text not correct'
      assert x.getUrlName()==urlName, 'Url name not correct'
      assert x.getDescription()==description, 'Description not correct'
      import time
      assert time.time()-x.getDate()<1, 'Invalid hotspot date'
      x.editData(coords=coordinates, url='')
      assert x.getUrl()==url, 'Url not correct'
      x.editData(coords=coordinates, url=None)
      assert x.getUrl()=='', 'Url not correct'
      x.move(10,15)
      assert(x.getCoordinates()==(30,35,55,60)),'Annotation not moved correctly'
      x.moveTo(50,70)
      assert(x.getCoordinates()==(50,70,75,95)),'Annotation not moved correctly'
      assert(x._width==25 and x._height==25), 'Annotation dimensions incorrect'
      oldCoords=self.app.testMade.testImage.getAnnotation(self.annoText).getCoordinates()
      self.app.testMade.testImage.updateAnnotationPosition(self.annoText, 10, 20, level=1,absolute='false')
      newCoords=self.app.testMade.testImage.getAnnotation(self.annoText).getCoordinates()
      assert(newCoords[2]==oldCoords[0]+10*im_mag+(oldCoords[2]-oldCoords[0])),\
          'Annotation location updating failed'
      assert (x.getKeywords()=='test1 test2 test3_test4_test5 test6'+\
          ' test7_test8 test9_test10 test11 test12 test13'),\
          'Annotation keywords invalid' 
      assert (x.getKeywordList()=='test1, test2, "test3 test4 test5", test6,'+\
          ' "test7 test8",  "test9 test10", test11, test12, test13'),\
          'Annotation keywordlist invalid'
      assert (x.getKeywordList2()=='test1 test2 "test3 test4 test5"'+\
          ' test6 "test7 test8"  "test9 test10" test11 test12 test13'),\
          'Annotation keywordlist2 invalid'

    def testDelete(self):
      made.REQUEST=self.app.REQUEST
      self.app.REQUEST.next='testMade'
      made.testImage.removeAnnotation(self.annoText)
      assert made.testImage.getAnnotationsForPoint(5,5)==None,\
          'Annotation not removed'
      assert str(self.app.REQUEST.RESPONSE).find('Location: testMade?level=3&')\
          !=-1,'Invalid redirect after annotation removal'
      assert self.annoText not in made.testImage.getAnnotationsList().keys(),\
          'Annotation not removed'

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestAnnotations))
    return suite

if __name__ == '__main__':
    framework()

